from rpi2mqtt.base import Sensor
import json
from datetime import datetime, timedelta


class Scanner(Sensor):

    def __init__(self, name, topic, beacon_uuid, away_timeout=10):
        super(Scanner, self).__init__(name, None, topic, 'presence', 'ibeacon')
        self.present = 'OFF'
        self.rssi = None
        self.beacon_uuid = beacon_uuid
        self.away_timeout = away_timeout
        self.last_seen = datetime.now()
        self.setup()

    @property
    def homeassistant_mqtt_config(self):
        config = super(Scanner, self).homeassistant_mqtt_config
        config['value_template'] = "{{ value_json.presence }}"
        # config['command_topic'] = self.topic + '/set'
        return config

    @property
    def homeassistant_mqtt_config_topic(self):
        return 'homeassistant/binary_sensor/{}_{}/config'.format(self.name, self.device_class)

    def setup(self):
        """
        Setup Home Assistant MQTT discover for ibeacons.
        :return: None
        """
        # device_config = {'name': "iBeacon",
        #                  'identifiers': self.name,
        #                  'sw_version': 'rpi2mqtt',
        #                  'model': "iBeacon",
        #                  'manufacturer': 'Generic'}

        # config = json.dumps({'name': self.name + '_ibeacon',
        #                      'device_class': 'presence',
        #                      'value_template': "{{ value_json.presence }}",
        #                      'unique_id': self.name + '_ibeacon_rpi2mqtt',
        #                      'state_topic': self.topic,
        #                      "json_attributes_topic": self.topic,
        #                      'device': device_config})

        # mqtt.publish('homeassistant/binary_sensor/{}_{}/config'.format(self.name, 'presence'), config)

    def process_ble_update(self, bt_addr, rssi, packet, additional_info):
        new_state = self.present
        if self.beacon_uuid == additional_info['uuid']:
            new_state = 'ON'
        self.rssi = rssi
        self.last_seen = datetime.now()

        if new_state != self.present:
            self.present = new_state
            self.callback()

    def state(self):
        if self.present == 'ON' and self.last_seen + timedelta(seconds=self.away_timeout) < datetime.now():
            self.present = 'OFF'

        return self.present

    def payload(self):
        return json.dumps({'presence': self.present, 'rssi': self.rssi})

    # def callback(self):
    #     mqtt.publish(self.topic, self.payload())


import logging
import sys
logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)

from rpi2mqtt.mqtt import MQTT 
import time


def main():
    logging.debug('debug message')
    logging.info('info message')
    logging.warning('warning')
    logging.error('error')
    MQTT.setup()

def test_mqtt_subscribe():
    MQTT.setup()

    def callback(client, data, message):
        print(message.payload)

    while True:
        res, id = MQTT.subscribe('hello', callback)
        if res == 0:
            break
        time.sleep(1)

    while True:
        pass
        # mqtt.setup()

if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
    test_mqtt_subscribe()



import logging
import sys
import yaml
from dotmap import DotMap
import pathlib
logging.basicConfig(stream=sys.stdout, format='%(asctime)s:%(levelname)s:%(message)s')


class Config():
    """Config holder singleton. Use Config.get_instance() in submodules to access. Initial call must include 'filename'.

    Attributes:
        [type]: [description]
    """
    _config = None
    _filename = None

    @classmethod
    def get_instance(cls, filename=None):
        if filename:
            cls._filename = filename

        if not cls._config and cls._filename:
            with open(cls._filename, 'r') as f:
                cls._config = DotMap(yaml.safe_load(f))
                config_filename = pathlib.Path(f.name).absolute()
                logging.info("Loaded config file {}".format(config_filename))
            Config.set_log_level(cls._config.loglevel)
        return cls._config

    @staticmethod
    def set_log_level(level):
        _level = logging.WARN
        if level == 'info':
            _level = logging.INFO
        elif level == 'debug':
            _level = logging.DEBUG
        elif level == 'error':
            _level = logging.ERROR
        logger = logging.getLogger().setLevel(_level)

    @classmethod
    def save(cls):
        logging.info("Saving config file...")
        with open(cls._filename, 'w') as f:
            yaml.dump(cls._config, f)


def generate_config(config_filename):
    with open(config_filename, 'w') as f: 
        default_config = {
            "logging": 'warn',
            'polling_interval': 300,
            'mqtt': {
                'host': 'hostname',
                'port': 1883,
                'username': 'mqtt_user',
                'password': 'mqtt_password',
                'retries': 3,
                'ca_cert': '/path/to/certificate'
            },
            'sensors': [
                {'name': 'sensor_name', 'type': 'sensor_type', 'pin': 'gpio pin number'}
            ]}
        f.write(yaml.dump(default_config))




# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# from setuptools.command.install import install as _install
# To use a consistent encoding
# from codecs import open
# from os import path
# import sys
# import os

# Get the long description from the relevant file
# with open('README.rst', encoding='utf-8') as f:
#     long_description = f.read()

setup(
    name='rpi2mqtt',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.5.7',

    # description='A Jupyter notebook extension to hide code, prompts and outputs.',
    # long_description=long_description,

    # The project's main homepage.
    url='https://github.com/kirbs-/rpi2mqtt',

    # Author details
    author='Chris Kirby',
    author_email='kirbycm@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],

    # What does your project relate to?
    keywords='raspberrypi mqtt GPIO home_automation home_assistant',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    # packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    packages={'rpi2mqtt'},

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'dotmap', 
        'pyyaml', 
        'paho-mqtt', 
        'Adafruit_DHT', 
        'RPi.bme280', 
        'pathlib', 
        'smbus2', 
        'beacontools', 
        'RPi.GPIO',
        'pendulum', 
        'poetry',
        'schedule'
    ],
    entry_points={
        'console_scripts': ['rpi2mqtt=rpi2mqtt.event_loop:main']
    }
)
